@@ -1,8 +1,13 @@
-# $Author: ddumont $
-# $Date: 2009-04-17 13:39:18 +0200 (ven 17 avr 2009) $
-# $Revision: 935 $
-
-#    Copyright (c) 2005-2009 Dominique Dumont.
+# 
+# This file is part of Config-Model
+# 
+# This software is Copyright (c) 2010 by Dominique Dumont.
+# 
+# This is free software, licensed under:
+# 
+#   The GNU Lesser General Public License, Version 2.1, February 1999
+# 
+#    Copyright (c) 2005-2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -30,17 +35,17 @@ use Config::Model::Dumper;
 use Config::Model::DumpAsData;
 use Config::Model::Report;
 use Config::Model::Describe;
-# use Log::Log4perl ;
+use Log::Log4perl qw(get_logger :levels);
 use UNIVERSAL;
 use Scalar::Util qw/weaken/;
 use Storable qw/dclone/ ;
 
 use base qw/Config::Model::AutoRead/;
 
-use vars qw($VERSION $AUTOLOAD @status @level
+use vars qw($AUTOLOAD @status @level
 @experience_list %experience_index );
 
-$VERSION = sprintf "1.%04d", q$Revision: 935 $ =~ /(\d+)/;
+our $VERSION="1.201";
 
 *status           = *Config::Model::status ;
 *level            = *Config::Model::level ;
@@ -53,10 +58,16 @@ my %legal_properties = (
 			experience => {qw/master 1 advanced 1 beginner 1/},
 		       ) ;
 
+my $logger = get_logger("Tree::Node") ;
+
 =head1 NAME
 
 Config::Model::Node - Class for configuration tree node
 
+=head1 VERSION
+
+version 1.205
+
 =head1 SYNOPSIS
 
  $model->create_config_class 
@@ -211,13 +222,13 @@ See L<Config::Model::AutoRead> for details.
 =head2 Element type
 
 Each element is declared with a list ref that contains all necessary
-informations:
+information:
 
   element => [ 
                foo => { ... }
              ]
 
-This most important informations from this hash ref is the mandatory 
+This most important information from this hash ref is the mandatory
 B<type> parameter. The I<type> type can be:
 
 =cut
@@ -559,7 +570,7 @@ sub new {
       ? $self->{parent}->name : 'user' ;
 
     my $class_name = $self->{config_class_name} ;
-    print "New $class_name requested by $caller_class\n" if $::verbose;
+    $logger->info( "New $class_name requested by $caller_class");
 
     my $model 
       = $self->{model} 
@@ -748,13 +759,10 @@ C<hash>, C<leaf>,...). By default return elements of any type.
 =item *
 
 B<cargo_type>: Returns only element which contain requested type.
- E.g. if C<get_element_name> is called with C<< cargo_type => leaf >>,
- C<get_element_name> will return simple leaf elements, but also hash
- or list element that contain L<leaf|Config::Model::Value> object. By
- default return elements of any type.
-
-
-
+E.g. if C<get_element_name> is called with C<< cargo_type => leaf >>,
+C<get_element_name> will return simple leaf elements, but also hash
+or list element that contain L<leaf|Config::Model::Value> object. By
+default return elements of any type.
 
 =back
 
@@ -810,8 +818,7 @@ sub get_element_name {
 	}
     }
 
-    print "get_element_name: got @result for level $for\n"
-      if $::debug ;
+    $logger->debug( "get_element_name: got @result for level $for");
 
     return wantarray ? @result : join( ' ', @result );
 }
@@ -829,18 +836,42 @@ sub next_element {
     my $self      = shift;
     my $element   = shift;
     my $min_experience = shift;
+    my $find_previous = shift || 0 ;
 
-    my @elements = $self->get_element_name(for => $min_experience);
+    my @elements = @{$self->{model}{element_list}} ;
+    @elements = reverse @elements if $find_previous ;
 
-    return $elements[0] unless defined $element and $element ;
+    # if element is empty, start from first element
+    my $found_elt = (defined $element and $element) ? 0 : 1 ;
 
-    my $i     = 0;
-    while (@elements) {
-        croak "next_element: element $element is unknown. Expected @elements"
-            unless defined $elements[$i];
-        last if $element eq $elements[ $i++ ];
+    while (my $name = shift @elements) {
+      if ($found_elt) {
+	return $name 
+	  if $self->is_element_available(name => $name, 
+					 experience => $min_experience);
+      }
+      $found_elt = 1 if $element eq $name ;
     }
-    return $elements[$i];
+
+    croak "next_element: element $element is unknown. Expected @elements" 
+      unless $found_elt;
+    return;
+}
+
+=head2 previous_element ( element_name, [ experience_index ] )
+
+This method provides a way to iterate through the elements of a node.
+
+Returns the previous element name for a given experience (default
+C<master>).  Returns undef if no previous element is available.
+
+=cut
+
+sub previous_element {
+    my $self      = shift;
+    my $element   = shift;
+    my $min_experience = shift;
+    $self->next_element($element,$min_experience,1) ;
 }
 
 =head2 get_element_property ( element => ..., property => ... )
@@ -885,8 +916,7 @@ sub set_element_property {
     my $new_value = $args{value} || 
       croak "set_element_property:: missing 'value' parameter";
 
-    print "Node ",$self->name,": set $elt property $prop to $new_value\n"
-      if $::debug;
+    $logger->debug("Node ",$self->name,": set $elt property $prop to $new_value");
 
    return $self->{model}{$prop}{$elt} = $new_value ;
 }
@@ -910,9 +940,8 @@ sub reset_element_property {
 			       %args
 			      );
 
-    print "Node ",$self->name,
-      ": reset $elt property $prop to $original_value\n"
-	if $::debug;
+    $logger->debug( "Node ",$self->name,
+      ": reset $elt property $prop to $original_value");
 
     return $self->{model}{$prop}{$elt} = $original_value ;
 }
@@ -1251,27 +1280,31 @@ sub load_data {
     my $self = shift ;
     my $p    = shift ;
 
+    my $check = $self->instance->get_value_check('store') ;
+
     if ( not defined $p or (ref($p) ne 'HASH' and not $p->isa( 'HASH' )) ) {
 	Config::Model::Exception::LoadData
 	    -> throw (
 		      object => $self,
 		      message => "load_data called with non hash ref arg",
 		      wrong_data => $p,
-		     )  if $self->instance->get_value_check('store');
+		     )  if $check ;
 	return ;
     }
 
     my $h = dclone $p ;
 
-    print "Node load_data (",$self->location,") will load elt ",
-      join (' ',keys %$h),"\n" if $::verbose ;
+    $logger->info("Node load_data (",$self->location,") will load elt ",
+		  join (' ',keys %$h));
 
     # data must be loaded according to the element order defined by
     # the model
     foreach my $elt ( @{$self->{model}{element_list}} ) {
 	next unless defined $h->{$elt} ;
-	if ($self->is_element_available(name => $elt, experience => 'master')) {
-	    my $obj = $self->fetch_element($elt) ;
+	if ($self->is_element_available(name => $elt, experience => 'master')
+	    or not $check
+	   ) {
+	    my $obj = $self->fetch_element($elt,'master', not $check) ;
 	    $obj -> load_data(delete $h->{$elt}) ;
 	}
 	else {
@@ -1281,11 +1314,11 @@ sub load_data {
                                    . "element '$elt' with",
 			  wrong_data => $h->{$elt},
 			  object => $self,
-			 ) if $self->instance->get_value_check('store');
+			 ) ;
 	}
     }
 
-    if (%$h and $self->instance->get_value_check('store')) {
+    if (%$h and $check) {
 	Config::Model::Exception::LoadData 
 	    -> throw (
 		      message => "load_data: unknown elements (expected "
@@ -1296,9 +1329,10 @@ sub load_data {
     }
 }
 
-=head2 dump_tree ( [ mode => custom | full ] )
+=head2 dump_tree ( ... )
 
-Dumps the configuration data of the node and its siblings into a string.
+Dumps the configuration data of the node and its siblings into a
+string.  See L<Config::Model::Dumper/dump_tree> for parameter details.
 
 This string follows the syntax defined in
 L<Config::Model::Loader>. The string produced by C<dump_tree> can be
@@ -1371,7 +1405,7 @@ sub copy_from {
     $self->instance->push_no_value_check('fetch') ;
     my $dump = $from->dump_tree() ;
     $self->instance->pop_no_value_check ;
-    print "node copy with '$dump'\n" if $::debug ;
+    $logger->debug( "node copy with '$dump'");
     $self->load( step => $dump, check_store => 0 ) ;
 }
 
@@ -1422,7 +1456,7 @@ sub get_help {
 
 As configuration model are getting bigger, the load time of a tree
 gets longer. The L<Config::Model::AutoRead> class provides a way to
-load the configuration informations only when needed.
+load the configuration information only when needed.
 
 TBD
 
@@ -1440,5 +1474,4 @@ L<Config::Model::CheckList>,
 L<Config::Model::WarpedNode>,
 L<Config::Model::Value>
 
-=cut
-
+=cut
\ No newline at end of file
